/******************************************************************************
 * Copyright (C) 2017 Xilinx, Inc.  All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Use of the Software is limited solely to applications:
 * (a) running on a Xilinx device, or
 * (b) that interact with a Xilinx device through a bus or interconnect.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of the Xilinx shall not be used
 * in advertising or otherwise to promote the sale, use or other dealings in
 * this Software without prior written authorization from Xilinx.
 ******************************************************************************/

#include "xpfw_default.h"
#include "xpfw_config.h"
#include "xpfw_core.h"
#include "xpfw_events.h"
#include "xpfw_module.h"

#include "xpfw_ipi_manager.h"
#include "xpfw_mod_ipi_example.h"

/* 10 Seconds */
#define XPFW_IPI_MSG_SEND_TIME		10000U
#define IPI_PMU_CH0_MASK			XPAR_XIPIPS_TARGET_PSU_PMU_0_CH0_MASK
#define RESP_AND_MSG_NUM_OFFSET		0x1U
#define IPI_HEADER_OFFSET			0x0U

const XPfw_Module_t *IpiExampleModPtr;

static void IpiExampleHandler(const XPfw_Module_t *ModPtr, u32 IpiNum, u32 SrcMask, const u32* Payload, u8 Len)
{
	u32 RespBuf[2] = {0};

	if (IpiNum > 0) {
		XPfw_Printf(DEBUG_ERROR,"PMU: IPI Example Handler: It handles"
				" only IPI on PMU-0\r\n");
	} else {
		XPfw_Printf(DEBUG_DETAILED, "PMU: Payload received:\r\n");
		xil_printf("PMU: IPI Message number from payload: 0x%x\r\n",
				Payload[RESP_AND_MSG_NUM_OFFSET]);

		RespBuf[RESP_AND_MSG_NUM_OFFSET] = XST_SUCCESS;
		XPfw_IpiWriteResponse(ModPtr, SrcMask, RespBuf, 2);
	}
}

static void XPfw_SendIpi(void)
{
	s32 Status;
	static u32 MsgPtr[2] = {0x0};
	u32 RespPtr[2] = {0};

	MsgPtr[1] += 1;
	Status = XPfw_IpiWriteMessage(IpiExampleModPtr, IPI_PMU_0_IER_RPU_0_MASK,
			MsgPtr, sizeof(MsgPtr)/sizeof(MsgPtr[1]));

	if(XST_SUCCESS != Status) {
		XPfw_Printf(DEBUG_ERROR, "PMU: IPI Write Message failed\r\n");
		goto END;
	}

	Status = XPfw_IpiTrigger(IPI_PMU_0_IER_RPU_0_MASK);

	if(XST_SUCCESS != Status) {
		XPfw_Printf(DEBUG_ERROR, "PMU: IPI Trigger failed\r\n");
		goto END;
	}

	Status = XPfw_IpiPollForAck(IPI_PMU_0_IER_RPU_0_MASK, (~0));

	if(XST_SUCCESS != Status) {
		XPfw_Printf(DEBUG_ERROR, "PMU: IPI Poll for Ack failed\r\n");
		goto END;
	}

	Status = XPfw_IpiReadResponse(IpiExampleModPtr, IPI_PMU_0_IER_RPU_0_MASK,
			RespPtr, sizeof(RespPtr)/sizeof(RespPtr[1]));

	if(XST_SUCCESS != Status) {
		XPfw_Printf(DEBUG_ERROR, "PMU: IPI Read Response failed\r\n");
		goto END;
	}

	xil_printf("PMU: Response received.\r\n");
	xil_printf("PMU: IPI Response header: 0x%x\r\n", RespPtr[IPI_HEADER_OFFSET]);
	xil_printf("PMU: IPI Response status: 0x%x\r\n", RespPtr[RESP_AND_MSG_NUM_OFFSET]);
END:
	XPfw_Printf(DEBUG_DETAILED,"");
}

static void IpiExampleCfgInit(const XPfw_Module_t *ModPtr, const u32 *CfgData, u32 Len)
{
	s32 Status;

	Status = XPfw_CoreScheduleTask(ModPtr, XPFW_IPI_MSG_SEND_TIME, XPfw_SendIpi);
	if (XST_FAILURE == Status) {
		XPfw_Printf(DEBUG_ERROR,"PMU: (MOD-%d):Scheduling Send IPI task "
				"failed\r\n", ModPtr->ModId);
	}
}

void ModIpiExampleInit(void)
{
	IpiExampleModPtr = XPfw_CoreCreateMod();

	if (XPfw_CoreSetCfgHandler(IpiExampleModPtr, IpiExampleCfgInit) != XST_SUCCESS) {
		XPfw_Printf(DEBUG_DETAILED,
				"PMU: Warning: IpiExampleModPtr: Failed to set CfgHandler \r\n")
	}
	(void)XPfw_CoreSetIpiHandler(IpiExampleModPtr, IpiExampleHandler, 0x1EU);

}

